package org.artifactory.servicetrust;

import org.artifactory.api.security.access.TokenSpec;
import org.jfrog.access.common.ServiceId;
import org.jfrog.access.token.JwtAccessToken;

/**
 * @author igoru
 */
public interface TokenSpecService {

    long DEFAULT_PAIRING_TOKEN_EXPIRATION_TIME_IN_SECS = 5 * 60L;
    long DEFAULT_MASTER_TOKEN_EXPIRATION_TIME_IN_SECS = 365 * 24 * 60 * 60L;
    long DEFAULT_EXCHANGE_TOKEN_EXPIRATION_TIME_IN_SECS = 60 * 60L;
    String INTERNAL_SCOPE_PREFIX = "internal:";
    String INTERNAL_SERVICE_TRUST_PAIRING_SCOPE_PREFIX = INTERNAL_SCOPE_PREFIX + "service-trust/pairing/";
    String INTERNAL_SERVICE_TRUST_PAIRING_SCOPE_POSTFIX = ":x";
    String SCOPE_PATTERN =
            INTERNAL_SERVICE_TRUST_PAIRING_SCOPE_PREFIX + "%s" + INTERNAL_SERVICE_TRUST_PAIRING_SCOPE_POSTFIX;

    TokenSpec createPairingTokenSpec(ServiceId artifactoryServiceId, ServiceTrust trust, String subject, String scope,
            String baseUrl, String pairingUrl,
            String exchangeUrl);

    TokenSpec createMasterTokenSpec(ServiceId artifactoryServiceId, String useCasePath, String pairingScope,
            String baseUrl, String exchangeUrl);

    TokenSpec createExchangeTokenSpec(ServiceId artifactoryServiceId, String useCasePath);

    ExchangeTokenRequest createExchangeTokenRequest(String subject, String useCasePath,
            JwtAccessToken jwt);

    /**
     * Store master token together with exchange url, base url and other
     */
    void storeMasterTokenData(ExchangeTokenResponse response, TokenExtension extension);

    /**
     * Update master token to avoid expiration
     */
    void updateMasterTokenData(ExchangeTokenResponse response);

    /**
     * Create exchange token to refresh master token
     */
    ExchangeTokenRequest createMasterTokenExchangeRequest(String subject);

    ExchangeTokenRequest createScopedTokenExchangeRequest(String subject);
}
