package org.artifactory.traffic.entry;

import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.artifactory.traffic.entry.TokenizedTrafficEntry.COLUMN_SEPARATOR;
import static org.testng.Assert.assertTrue;

/**
 * @author Andrey Tashlitsky
 */
public class OutboundRequestEntryTest {

    @Test(dataProvider = "testOutboundRequestEntryToStringDataProvider")
    public void testOutboundRequestEntryToString(OutboundRequestEntry outboundRequestEntry, String expectedString) {
        assertTrue(outboundRequestEntry.toString().endsWith(expectedString));
    }

    @DataProvider
    public static Object[][] testOutboundRequestEntryToStringDataProvider() {
        return new Object[][] {
                {OutboundRequestEntry.builder().repoName("cool-repo").userName("cooluser").method("GET")
                        .url("http://cool.com/").statusCode(200).requestContentLength(10)
                        .responseContentLength(50).duration(324).build(),
                        COLUMN_SEPARATOR + "cool-repo" + COLUMN_SEPARATOR + "cooluser" + COLUMN_SEPARATOR + "GET" +
                                COLUMN_SEPARATOR + "http://cool.com/" + COLUMN_SEPARATOR + "200" +
                                COLUMN_SEPARATOR + "10" + COLUMN_SEPARATOR + "50" + COLUMN_SEPARATOR + "324"},
                {OutboundRequestEntry.builder().repoName("cool-repo").userName(null).method("HEAD")
                        .url("http://cool.com/cool/page").statusCode(404).requestContentLength(0)
                        .responseContentLength(0).duration(3242).build(),
                        COLUMN_SEPARATOR + "cool-repo" + COLUMN_SEPARATOR + "" + COLUMN_SEPARATOR + "HEAD" +
                                COLUMN_SEPARATOR + "http://cool.com/cool/page" + COLUMN_SEPARATOR + "404" +
                                COLUMN_SEPARATOR + "0" + COLUMN_SEPARATOR + "0" + COLUMN_SEPARATOR + "3242"}
        };
    }
}
