package org.artifactory.traffic.v2;

import org.artifactory.traffic.entry.DownloadEntry;
import org.artifactory.traffic.entry.RedirectEntry;
import org.artifactory.traffic.entry.TransferEntry;
import org.artifactory.traffic.entry.UploadEntry;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.testng.Assert.assertTrue;

public class TrafficEntryV2Test {

    @DataProvider
    public static Object[][] entries() {
        return new Object[][] {
                {
                        new DownloadEntry("repoPath", 48, 5,
                                "userAddress", "userIdenifier"),
                        "|DOWNLOAD|USER||userAddress|repoPath|48|48|48||"
                },
                {
                        new RedirectEntry("repoPath2", 49, 6,
                                "userAddress2"),
                        "|REDIRECT|USER||userAddress2|repoPath2|49|49|49||"
                },
                {
                        new UploadEntry("repoPath3", 50, 7,
                                "userAddress3"),
                        "|UPLOAD|USER||userAddress3|repoPath3|50|50|50||"
                },
                {
                        new DownloadEntry("repoPath4", 51, 8,
                                "userAddress4", "userxray"),
                        "|DOWNLOAD|XRAY||userAddress4|repoPath4|51|51|51||"
                }
        };
    }

    @Test(dataProvider = "entries")
    public void test(TransferEntry entry, String expected) {
        TrafficEntryV2 trafficEntryV2 = TrafficEntryV2.map(entry);
        String string = trafficEntryV2.toString();
        assertTrue(string.endsWith(expected));
    }
}
