package org.artifactory.version.converter.v302;

import org.artifactory.version.converter.XmlConverter;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.Namespace;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

/**
 * @author Igor Usenko
 */
public class FixNpmDefaultLayoutConverter implements XmlConverter {
    private static final Logger log = LoggerFactory.getLogger(FixNpmDefaultLayoutConverter.class);
    private static final String FIXED_PATH_PATTERN = "[orgPath]/-/[module]-[baseRev](-[fileItegRev]).tgz";
    private static final String ARTIFACT_PATH_PATTERN = "artifactPathPattern";

    @Override
    public void convert(Document doc) {
        log.info("Starting 'Npm fix default repository layout' conversion");
        Element rootElement = doc.getRootElement();
        Namespace namespace = rootElement.getNamespace();

        Element repoLayoutsElement = rootElement.getChild("repoLayouts", namespace);
        List<Element> repoLayouts = repoLayoutsElement.getChildren();

        for (Element layout : repoLayouts) {
            Element nameElement = layout.getChild("name", namespace);
            String name = nameElement.getText();

            if ("npm-default".equals(name)) {
                log.debug("Found npm-default layout");
                Element pathPatternElement = layout.getChild(ARTIFACT_PATH_PATTERN, namespace);
                pathPatternElement.setText(FIXED_PATH_PATTERN);
                log.debug("Fixed npm-default layout successfully");
            }
        }
        log.info("Finished 'Npm fix default repository layout' conversion successfully");
    }

}