package org.artifactory.storage.db.search.model;

import org.testng.annotations.Test;

import java.util.Arrays;
import java.util.List;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

/**
 * @author Yevdo Abramov
 * Created on 30/12/2019
 */
public class VfsQueryCriterionDbImplTest {

    @Test(testName = "Test criteria builder for single string value")
    public void testCriteriaBuilderForSingleValue() {
        VfsQueryCriterionDbImpl criterion = new VfsQueryCriterionDbImpl("node_name");
        criterion.setValue("MyArt");
        DbSqlQueryBuilder queryBuilder = new DbSqlQueryBuilder();
        queryBuilder.append("SELECT node_name FROM node WHERE ");
        criterion.fill(queryBuilder);
        assertEquals(queryBuilder.toString(), "SELECT node_name FROM node WHERE nodes.node_name = 'MyArt' ");
    }

    @Test(testName = "Test case insensitive criteria builder for single string value")
    public void testCaseInsensitiveCriteriaBuilderForSingleValue() {
        VfsQueryCriterionDbImpl criterion = new VfsQueryCriterionDbImpl("node_name");
        criterion.setCaseInsensitive(true);
        criterion.setValue("MyArt");
        DbSqlQueryBuilder queryBuilder = new DbSqlQueryBuilder();
        queryBuilder.append("SELECT node_name FROM node WHERE ");
        criterion.fill(queryBuilder);
        assertEquals(queryBuilder.toString(), "SELECT node_name FROM node WHERE lower(nodes.node_name) = 'myart' ");
    }

    @Test(testName = "Test case insensitive criteria builder for iterable value")
    public void testCaseInsensitiveCriteriaBuilderForIterableValue() {
        VfsQueryCriterionDbImpl criterion = new VfsQueryCriterionDbImpl("node_name");
        criterion.setCaseInsensitive(true);
        criterion.setValue(Arrays.asList("MyArt", "TheArt"));
        DbSqlQueryBuilder queryBuilder = new DbSqlQueryBuilder();
        queryBuilder.append("SELECT node_name FROM node WHERE ");
        criterion.fill(queryBuilder);
        assertEquals(queryBuilder.sqlQuery.toString(),
                "SELECT node_name FROM node WHERE lower(nodes.node_name) = (#) ");

        List params = (List) queryBuilder.params.get(0);
        assertEquals(params.size(), 2);
        assertTrue(params.contains("myart"));
        assertTrue(params.contains("theart"));
    }
}
