package org.artifactory.ui.rest.service.onboarding;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.eula.EulaService;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.security.UserGroupService;
import org.artifactory.common.config.db.ArtifactoryCommonDbPropertiesService;
import org.artifactory.common.storage.db.properties.DbVersionInfo;
import org.artifactory.security.access.AccessService;
import org.artifactory.version.ArtifactoryVersionProvider;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import static org.mockito.Mockito.when;

/**
 * @author gavriels
 */
@Test
public class GetArtifactoryInitStatusServiceTest {

    @Mock
    private AddonsManager addonsManager;
    @Mock
    private CentralConfigService configService;
    @Mock
    private UserGroupService userGroupService;
    @Mock
    private AccessService accessService;
    @Mock
    private EulaService eulaService;
    @Mock
    private ArtifactoryCommonDbPropertiesService artifactoryCommonDbPropertiesService;

    private GetArtifactoryInitStatusService getArtifactoryInitStatusService;

    @BeforeMethod
    public void init() {
        MockitoAnnotations.initMocks(this);
        getArtifactoryInitStatusService = new GetArtifactoryInitStatusService(
                addonsManager,
                configService,
                userGroupService,
                accessService,
                eulaService,
                artifactoryCommonDbPropertiesService);
    }

    @Test
    public void testShouldSkipOnboardingWizard() {
        List<DbVersionInfo> dbVersionInfos = CreateDbVersionListForTest();
        when(artifactoryCommonDbPropertiesService.getAllDbVersionInfo()).thenReturn(dbVersionInfos.subList(0, 2));
        Assert.assertFalse(getArtifactoryInitStatusService.shouldSkipOnboardingWizard());
        when(artifactoryCommonDbPropertiesService.getAllDbVersionInfo()).thenReturn(dbVersionInfos.subList(0, 3));
        Assert.assertTrue(getArtifactoryInitStatusService.shouldSkipOnboardingWizard());
        dbVersionInfos.remove(2);
        when(artifactoryCommonDbPropertiesService.getAllDbVersionInfo()).thenReturn(dbVersionInfos);
        Assert.assertTrue(getArtifactoryInitStatusService.shouldSkipOnboardingWizard());
    }

    private List<DbVersionInfo> CreateDbVersionListForTest() {
        return new ArrayList<>(
                Arrays.asList(
                        new DbVersionInfo(new Date().getTime(),
                                "7.20.0",
                                72000900,
                                1111),
                        new DbVersionInfo(new Date().getTime(),
                                ArtifactoryVersionProvider.v7100m001.get().getVersion(),
                                (int) ArtifactoryVersionProvider.v7100m001.get().getRevision(),
                                1111),
                        new DbVersionInfo(new Date().getTime(),
                                ArtifactoryVersionProvider.v760.get().getVersion(),
                                (int) ArtifactoryVersionProvider.v760.get().getRevision(),
                                1111),
                        new DbVersionInfo(new Date().getTime(),
                                "7.8.15",
                                70815900,
                                1111))
        );
    }
}